@@ -1,8 +1,6 @@
 """
 Password generation for the Jupyter Server.
 """
-
-from contextlib import contextmanager
 import getpass
 import hashlib
 import io
@@ -11,17 +9,19 @@ import os
 import random
 import traceback
 import warnings
+from contextlib import contextmanager
 
-from ipython_genutils.py3compat import cast_bytes, str_to_bytes, cast_unicode
-from traitlets.config import Config, ConfigFileNotFound, JSONFileConfigLoader
 from jupyter_core.paths import jupyter_config_dir
+from traitlets.config import Config
+from traitlets.config import ConfigFileNotFound
+from traitlets.config import JSONFileConfigLoader
 
 # Length of the salt in nr of hex chars, which implies salt_len * 4
 # bits of randomness.
 salt_len = 12
 
 
-def passwd(passphrase=None, algorithm='argon2'):
+def passwd(passphrase=None, algorithm="argon2"):
     """Generate hashed password and salt for use in server configuration.
 
     In the server configuration, set `c.ServerApp.password` to
@@ -48,19 +48,20 @@ def passwd(passphrase=None, algorithm='argon2'):
 
     """
     if passphrase is None:
-        for i in range(3):
-            p0 = getpass.getpass('Enter password: ')
-            p1 = getpass.getpass('Verify password: ')
+        for _ in range(3):
+            p0 = getpass.getpass("Enter password: ")
+            p1 = getpass.getpass("Verify password: ")
             if p0 == p1:
                 passphrase = p0
                 break
             else:
-                print('Passwords do not match.')
+                print("Passwords do not match.")
         else:
-            raise ValueError('No matching passwords found. Giving up.')
+            raise ValueError("No matching passwords found. Giving up.")
 
-    if algorithm == 'argon2':
+    if algorithm == "argon2":
         import argon2
+
         ph = argon2.PasswordHasher(
             memory_cost=10240,
             time_cost=10,
@@ -68,13 +69,13 @@ def passwd(passphrase=None, algorithm='argon2'):
         )
         h = ph.hash(passphrase)
 
-        return ':'.join((algorithm, cast_unicode(h, 'ascii')))
+        return ":".join((algorithm, h))
 
     h = hashlib.new(algorithm)
-    salt = ('%0' + str(salt_len) + 'x') % random.getrandbits(4 * salt_len)
-    h.update(cast_bytes(passphrase, 'utf-8') + str_to_bytes(salt, 'ascii'))
+    salt = ("%0" + str(salt_len) + "x") % random.getrandbits(4 * salt_len)
+    h.update(passphrase.encode("utf-8") + salt.encode("ascii"))
 
-    return ':'.join((algorithm, salt, h.hexdigest()))
+    return ":".join((algorithm, salt, h.hexdigest()))
 
 
 def passwd_check(hashed_passphrase, passphrase):
@@ -105,9 +106,10 @@ def passwd_check(hashed_passphrase, passphrase):
     ...              'mypassword')
     True
     """
-    if hashed_passphrase.startswith('argon2:'):
+    if hashed_passphrase.startswith("argon2:"):
         import argon2
         import argon2.exceptions
+
         ph = argon2.PasswordHasher()
 
         try:
@@ -116,7 +118,7 @@ def passwd_check(hashed_passphrase, passphrase):
             return False
 
     try:
-        algorithm, salt, pw_digest = hashed_passphrase.split(':', 2)
+        algorithm, salt, pw_digest = hashed_passphrase.split(":", 2)
     except (ValueError, TypeError):
         return False
 
@@ -128,10 +130,11 @@ def passwd_check(hashed_passphrase, passphrase):
     if len(pw_digest) == 0:
         return False
 
-    h.update(cast_bytes(passphrase, 'utf-8') + cast_bytes(salt, 'ascii'))
+    h.update(passphrase.encode("utf-8") + salt.encode("ascii"))
 
     return h.hexdigest() == pw_digest
 
+
 @contextmanager
 def persist_config(config_file=None, mode=0o600):
     """Context manager that can be used to modify a config object
@@ -141,7 +144,7 @@ def persist_config(config_file=None, mode=0o600):
     """
 
     if config_file is None:
-        config_file = os.path.join(jupyter_config_dir(), 'jupyter_server_config.json')
+        config_file = os.path.join(jupyter_config_dir(), "jupyter_server_config.json")
 
     os.makedirs(os.path.dirname(config_file), exist_ok=True)
 
@@ -153,15 +156,15 @@ def persist_config(config_file=None, mode=0o600):
 
     yield config
 
-    with io.open(config_file, 'w', encoding='utf8') as f:
-        f.write(cast_unicode(json.dumps(config, indent=2)))
+    with io.open(config_file, "w", encoding="utf8") as f:
+        f.write(json.dumps(config, indent=2))
 
     try:
         os.chmod(config_file, mode)
-    except Exception as e:
+    except Exception:
         tb = traceback.format_exc()
-        warnings.warn("Failed to set permissions on %s:\n%s" % (config_file, tb),
-            RuntimeWarning)
+        warnings.warn("Failed to set permissions on %s:\n%s" % (config_file, tb), RuntimeWarning)
+
 
 def set_password(password=None, config_file=None):
     """Ask user for password, store it in JSON configuration file"""
