@@ -16,7 +16,6 @@ from http.cookies import Morsel
 from urllib.parse import urlparse
 
 import prometheus_client
-from ipython_genutils.path import filefind
 from jinja2 import TemplateNotFound
 from jupyter_core.paths import is_hidden
 from tornado import escape
@@ -31,6 +30,7 @@ from jupyter_server._tz import utcnow
 from jupyter_server.i18n import combine_translations
 from jupyter_server.services.security import csp_report_uri
 from jupyter_server.utils import ensure_async
+from jupyter_server.utils import filefind
 from jupyter_server.utils import url_escape
 from jupyter_server.utils import url_is_absolute
 from jupyter_server.utils import url_path_join
@@ -191,6 +191,10 @@ class AuthenticatedHandler(web.RequestHandler):
             return False
         return bool(self.login_handler.get_login_available(self.settings))
 
+    @property
+    def authorizer(self):
+        return self.settings.get("authorizer")
+
 
 class JupyterHandler(AuthenticatedHandler):
     """Jupyter-specific extensions to authenticated handling
@@ -251,7 +255,8 @@ class JupyterHandler(AuthenticatedHandler):
     @property
     def contents_js_source(self):
         self.log.debug(
-            "Using contents: %s", self.settings.get("contents_js_source", "services/contents")
+            "Using contents: %s",
+            self.settings.get("contents_js_source", "services/contents"),
         )
         return self.settings.get("contents_js_source", "services/contents")
 
@@ -309,7 +314,7 @@ class JupyterHandler(AuthenticatedHandler):
             self.set_header("Access-Control-Allow-Origin", self.allow_origin)
         elif self.allow_origin_pat:
             origin = self.get_origin()
-            if origin and self.allow_origin_pat.match(origin):
+            if origin and re.match(self.allow_origin_pat, origin):
                 self.set_header("Access-Control-Allow-Origin", origin)
         elif self.token_authenticated and "Access-Control-Allow-Origin" not in self.settings.get(
             "headers", {}
@@ -382,7 +387,7 @@ class JupyterHandler(AuthenticatedHandler):
         if self.allow_origin:
             allow = self.allow_origin == origin
         elif self.allow_origin_pat:
-            allow = bool(self.allow_origin_pat.match(origin))
+            allow = bool(re.match(self.allow_origin_pat, origin))
         else:
             # No CORS headers deny the request
             allow = False
@@ -427,7 +432,7 @@ class JupyterHandler(AuthenticatedHandler):
         if self.allow_origin:
             allow = self.allow_origin == origin
         elif self.allow_origin_pat:
-            allow = bool(self.allow_origin_pat.match(origin))
+            allow = bool(re.match(self.allow_origin_pat, origin))
         else:
             # No CORS settings, deny the request
             allow = False
@@ -449,7 +454,7 @@ class JupyterHandler(AuthenticatedHandler):
             return
         try:
             return super(JupyterHandler, self).check_xsrf_cookie()
-        except web.HTTPError as e:
+        except web.HTTPError:
             if self.request.method in {"GET", "HEAD"}:
                 # Consider Referer a sufficient cross-origin check for GET requests
                 if not self.check_referer():
@@ -548,13 +553,13 @@ class JupyterHandler(AuthenticatedHandler):
         if not self.request.body:
             return None
         # Do we need to call body.decode('utf-8') here?
-        body = self.request.body.strip().decode(u"utf-8")
+        body = self.request.body.strip().decode("utf-8")
         try:
             model = json.loads(body)
         except Exception as e:
             self.log.debug("Bad JSON: %r", body)
             self.log.error("Couldn't parse JSON", exc_info=True)
-            raise web.HTTPError(400, u"Invalid JSON in body of request") from e
+            raise web.HTTPError(400, "Invalid JSON in body of request") from e
         return model
 
     def write_error(self, status_code, **kwargs):
@@ -674,7 +679,8 @@ class APIHandler(JupyterHandler):
             )
         else:
             self.set_header(
-                "Access-Control-Allow-Headers", "accept, content-type, authorization, x-xsrftoken"
+                "Access-Control-Allow-Headers",
+                "accept, content-type, authorization, x-xsrftoken",
             )
         self.set_header("Access-Control-Allow-Methods", "GET, PUT, POST, PATCH, DELETE, OPTIONS")
 
