@@ -26,6 +26,7 @@ import sys
 import threading
 import time
 import urllib
+import warnings
 import webbrowser
 from base64 import encodebytes
 
@@ -74,7 +75,10 @@ from jupyter_server.services.kernels.kernelmanager import (
     AsyncMappingKernelManager,
 )
 from jupyter_server.services.config import ConfigManager
-from jupyter_server.services.contents.manager import AsyncContentsManager, ContentsManager
+from jupyter_server.services.contents.manager import (
+    AsyncContentsManager,
+    ContentsManager,
+)
 from jupyter_server.services.contents.filemanager import (
     AsyncFileContentsManager,
     FileContentsManager,
@@ -87,6 +91,7 @@ from jupyter_server.gateway.managers import (
     GatewaySessionManager,
     GatewayClient,
 )
+from jupyter_server.auth.authorizer import Authorizer, AllowAllAuthorizer
 
 from jupyter_server.auth.login import LoginHandler
 from jupyter_server.auth.logout import LogoutHandler
@@ -99,7 +104,6 @@ from jupyter_core.application import (
     base_flags,
     base_aliases,
 )
-from jupyter_core.paths import jupyter_config_path
 from jupyter_client import KernelManager
 from jupyter_client.kernelspec import KernelSpecManager
 from jupyter_client.session import Session
@@ -128,7 +132,6 @@ from jupyter_server.utils import (
     url_path_join,
     check_pid,
     url_escape,
-    pathname2url,
     unix_socket_in_use,
     urlencode_unix_socket_path,
     fetch,
@@ -168,7 +171,10 @@ JUPYTER_SERVICE_HANDLERS = dict(
         "jupyter_server.kernelspecs.handlers",
         "jupyter_server.services.kernelspecs.handlers",
     ],
-    nbconvert=["jupyter_server.nbconvert.handlers", "jupyter_server.services.nbconvert.handlers"],
+    nbconvert=[
+        "jupyter_server.nbconvert.handlers",
+        "jupyter_server.services.nbconvert.handlers",
+    ],
     security=["jupyter_server.services.security.handlers"],
     sessions=["jupyter_server.services.sessions.handlers"],
     shutdown=["jupyter_server.services.shutdown"],
@@ -191,7 +197,7 @@ def random_ports(port, n):
     """
     for i in range(min(5, n)):
         yield port + i
-    for i in range(n - 5):
+    for _ in range(n - 5):
         yield max(1, port + random.randint(-2 * n, 2 * n))
 
 
@@ -222,7 +228,16 @@ class ServerWebApplication(web.Application):
         default_url,
         settings_overrides,
         jinja_env_options,
+        authorizer=None,
     ):
+        if authorizer is None:
+            warnings.warn(
+                "authorizer unspecified. Using permissive AllowAllAuthorizer."
+                " Specify an authorizer to avoid this message.",
+                RuntimeWarning,
+                stacklevel=2,
+            )
+            authorizer = AllowAllAuthorizer(jupyter_app)
 
         settings = self.init_settings(
             jupyter_app,
@@ -237,6 +252,7 @@ class ServerWebApplication(web.Application):
             default_url,
             settings_overrides,
             jinja_env_options,
+            authorizer=authorizer,
         )
         handlers = self.init_handlers(default_services, settings)
 
@@ -256,6 +272,7 @@ class ServerWebApplication(web.Application):
         default_url,
         settings_overrides,
         jinja_env_options=None,
+        authorizer=None,
     ):
 
         _template_path = settings_overrides.get(
@@ -274,18 +291,16 @@ class ServerWebApplication(web.Application):
         )
         sys_info = get_sys_info()
 
-        # If the user is running the server in a git directory, make the assumption
-        # that this is a dev install and suggest to the developer `npm run build:watch`.
         base_dir = os.path.realpath(os.path.join(__file__, "..", ".."))
-        dev_mode = os.path.exists(os.path.join(base_dir, ".git"))
-
         nbui = gettext.translation(
-            "nbui", localedir=os.path.join(base_dir, "jupyter_server/i18n"), fallback=True
+            "nbui",
+            localedir=os.path.join(base_dir, "jupyter_server/i18n"),
+            fallback=True,
         )
         env.install_gettext_translations(nbui, newstyle=False)
 
         if sys_info["commit_source"] == "repository":
-            # don't cache (rely on 304) when working from master
+            # don't cache (rely on 304) when working from default branch
             version_hash = ""
         else:
             # reset the cache on server restart
@@ -314,7 +329,10 @@ class ServerWebApplication(web.Application):
                 "no_cache_paths": [url_path_join(base_url, "static", "custom")],
             },
             version_hash=version_hash,
+            # kernel message protocol over websoclet
+            kernel_ws_protocol=jupyter_app.kernel_ws_protocol,
             # rate limits
+            limit_rate=jupyter_app.limit_rate,
             iopub_msg_rate_limit=jupyter_app.iopub_msg_rate_limit,
             iopub_data_rate_limit=jupyter_app.iopub_data_rate_limit,
             rate_limit_window=jupyter_app.rate_limit_window,
@@ -335,6 +353,7 @@ class ServerWebApplication(web.Application):
             session_manager=session_manager,
             kernel_spec_manager=kernel_spec_manager,
             config_manager=config_manager,
+            authorizer=authorizer,
             # handlers
             extra_services=extra_services,
             # Jupyter stuff
@@ -356,6 +375,10 @@ class ServerWebApplication(web.Application):
 
         # allow custom overrides for the tornado web app.
         settings.update(settings_overrides)
+
+        if base_url and "xsrf_cookie_kwargs" not in settings:
+            # default: set xsrf cookie on base_url
+            settings["xsrf_cookie_kwargs"] = {"path": base_url}
         return settings
 
     def init_handlers(self, default_services, settings):
@@ -395,7 +418,7 @@ class ServerWebApplication(web.Application):
             # for each handler required for gateway, locate its pattern
             # in the current list and replace that entry...
             gateway_handlers = load_handlers("jupyter_server.gateway.handlers")
-            for i, gwh in enumerate(gateway_handlers):
+            for _, gwh in enumerate(gateway_handlers):
                 for j, h in enumerate(handlers):
                     if gwh[0] == h[0]:
                         handlers[j] = (gwh[0], gwh[1])
@@ -482,7 +505,6 @@ def shutdown_server(server_info, timeout=5, log=None):
     Returns True if the server was stopped by any means, False if stopping it
     failed (on Windows).
     """
-    from tornado.httpclient import HTTPClient, HTTPRequest
 
     url = server_info["url"]
     pid = server_info["pid"]
@@ -490,7 +512,7 @@ def shutdown_server(server_info, timeout=5, log=None):
     if log:
         log.debug("POST request to %sapi/shutdown", url)
 
-    r = fetch(url, method="POST", headers={"Authorization": "token " + server_info["token"]})
+    fetch(url, method="POST", headers={"Authorization": "token " + server_info["token"]})
     # Poll to see if it shut down.
     for _ in range(timeout * 10):
         if not check_pid(pid):
@@ -531,7 +553,7 @@ class JupyterServerStopApp(JupyterApp):
         help="Port of the server to be killed. Default %s" % DEFAULT_JUPYTER_SERVER_PORT,
     )
 
-    sock = Unicode(u"", config=True, help="UNIX socket of the server to be killed.")
+    sock = Unicode("", config=True, help="UNIX socket of the server to be killed.")
 
     def parse_command_line(self, argv=None):
         super(JupyterServerStopApp, self).parse_command_line(argv)
@@ -554,11 +576,11 @@ class JupyterServerStopApp(JupyterApp):
     def _maybe_remove_unix_socket(socket_path):
         try:
             os.unlink(socket_path)
-        except (OSError, IOError):
+        except OSError:
             pass
 
     def start(self):
-        servers = list(list_running_servers(self.runtime_dir))
+        servers = list(list_running_servers(self.runtime_dir, log=self.log))
         if not servers:
             self.exit("There are no running servers (per %s)" % self.runtime_dir)
         for server in servers:
@@ -576,7 +598,8 @@ class JupyterServerStopApp(JupyterApp):
                     return
         current_endpoint = self.sock or self.port
         print(
-            "There is currently no server running on {}".format(current_endpoint), file=sys.stderr
+            "There is currently no server running on {}".format(current_endpoint),
+            file=sys.stderr,
         )
         print("Ports/sockets currently in use:", file=sys.stderr)
         for server in servers:
@@ -619,7 +642,7 @@ class JupyterServerListApp(JupyterApp):
     )
 
     def start(self):
-        serverinfo_list = list(list_running_servers(self.runtime_dir))
+        serverinfo_list = list(list_running_servers(self.runtime_dir, log=self.log))
         if self.jsonlist:
             print(json.dumps(serverinfo_list, indent=2))
         elif self.json:
@@ -665,7 +688,10 @@ flags["autoreload"] = (
 # Add notebook manager flags
 flags.update(
     boolean_flag(
-        "script", "FileContentsManager.save_script", "DEPRECATED, IGNORED", "DEPRECATED, IGNORED"
+        "script",
+        "FileContentsManager.save_script",
+        "DEPRECATED, IGNORED",
+        "DEPRECATED, IGNORED",
     )
 )
 
@@ -724,6 +750,7 @@ class ServerApp(JupyterApp):
         GatewayKernelSpecManager,
         GatewaySessionManager,
         GatewayClient,
+        Authorizer,
     ]
     if terminado_available:  # Only necessary when terminado is available
         classes.append(TerminalManager)
@@ -762,7 +789,9 @@ class ServerApp(JupyterApp):
     @default("log_format")
     def _default_log_format(self):
         """override default log format to include date & time"""
-        return u"%(color)s[%(levelname)1.1s %(asctime)s.%(msecs).03d %(name)s]%(end_color)s %(message)s"
+        return (
+            "%(color)s[%(levelname)1.1s %(asctime)s.%(msecs).03d %(name)s]%(end_color)s %(message)s"
+        )
 
     # file to be opened in the Jupyter server
     file_to_run = Unicode("", help="Open the named file when the application is launched.").tag(
@@ -801,11 +830,15 @@ class ServerApp(JupyterApp):
     )
 
     allow_credentials = Bool(
-        False, config=True, help=_i18n("Set the Access-Control-Allow-Credentials: true header")
+        False,
+        config=True,
+        help=_i18n("Set the Access-Control-Allow-Credentials: true header"),
     )
 
     allow_root = Bool(
-        False, config=True, help=_i18n("Whether to allow the user to run the server as root.")
+        False,
+        config=True,
+        help=_i18n("Whether to allow the user to run the server as root."),
     )
 
     autoreload = Bool(
@@ -817,7 +850,9 @@ class ServerApp(JupyterApp):
     default_url = Unicode("/", config=True, help=_i18n("The default URL to redirect to from `/`"))
 
     ip = Unicode(
-        "localhost", config=True, help=_i18n("The IP address the Jupyter server will listen on.")
+        "localhost",
+        config=True,
+        help=_i18n("The IP address the Jupyter server will listen on."),
     )
 
     @default("ip")
@@ -841,12 +876,12 @@ class ServerApp(JupyterApp):
     @validate("ip")
     def _validate_ip(self, proposal):
         value = proposal["value"]
-        if value == u"*":
-            value = u""
+        if value == "*":
+            value = ""
         return value
 
     custom_display_url = Unicode(
-        u"",
+        "",
         config=True,
         help=_i18n(
             """Override URL shown to users.
@@ -866,7 +901,8 @@ class ServerApp(JupyterApp):
     port_default_value = DEFAULT_JUPYTER_SERVER_PORT
 
     port = Integer(
-        config=True, help=_i18n("The port the server will listen on (env: JUPYTER_PORT).")
+        config=True,
+        help=_i18n("The port the server will listen on (env: JUPYTER_PORT)."),
     )
 
     @default("port")
@@ -888,10 +924,12 @@ class ServerApp(JupyterApp):
     def port_retries_default(self):
         return int(os.getenv(self.port_retries_env, self.port_retries_default_value))
 
-    sock = Unicode(u"", config=True, help="The UNIX socket the Jupyter server will listen on.")
+    sock = Unicode("", config=True, help="The UNIX socket the Jupyter server will listen on.")
 
     sock_mode = Unicode(
-        "0600", config=True, help="The permissions mode for UNIX socket creation (default: 0600)."
+        "0600",
+        config=True,
+        help="The permissions mode for UNIX socket creation (default: 0600).",
     )
 
     @validate("sock_mode")
@@ -905,7 +943,7 @@ class ServerApp(JupyterApp):
                     bool(converted_value & stat.S_IRUSR),
                     bool(converted_value & stat.S_IWUSR),
                     # And isn't out of bounds.
-                    converted_value <= 2 ** 12,
+                    converted_value <= 2**12,
                 )
             )
         except ValueError:
@@ -917,17 +955,19 @@ class ServerApp(JupyterApp):
         return value
 
     certfile = Unicode(
-        u"", config=True, help=_i18n("""The full path to an SSL/TLS certificate file.""")
+        "",
+        config=True,
+        help=_i18n("""The full path to an SSL/TLS certificate file."""),
     )
 
     keyfile = Unicode(
-        u"",
+        "",
         config=True,
         help=_i18n("""The full path to a private key file for usage with SSL/TLS."""),
     )
 
     client_ca = Unicode(
-        u"",
+        "",
         config=True,
         help=_i18n(
             """The full path to a certificate authority certificate for SSL/TLS client authentication."""
@@ -974,7 +1014,9 @@ class ServerApp(JupyterApp):
                 f.write(secret)
         except OSError as e:
             self.log.error(
-                _i18n("Failed to write cookie secret to %s: %s"), self.cookie_secret_file, e
+                _i18n("Failed to write cookie secret to %s: %s"),
+                self.cookie_secret_file,
+                e,
             )
 
     token = Unicode(
@@ -1007,7 +1049,7 @@ class ServerApp(JupyterApp):
         if self.password:
             # no token if password is enabled
             self._token_generated = False
-            return u""
+            return ""
         else:
             self._token_generated = True
             return binascii.hexlify(os.urandom(24)).decode("ascii")
@@ -1036,7 +1078,9 @@ class ServerApp(JupyterApp):
             return DEFAULT_SOFT
 
         self.log.debug(
-            "Default value for min_open_files_limit is ignored (hard=%r, soft=%r)", hard, soft
+            "Default value for min_open_files_limit is ignored (hard=%r, soft=%r)",
+            hard,
+            soft,
         )
 
         return soft
@@ -1068,7 +1112,7 @@ class ServerApp(JupyterApp):
         self._token_generated = False
 
     password = Unicode(
-        u"",
+        "",
         config=True,
         help="""Hashed password to use for web authentication.
 
@@ -1213,7 +1257,7 @@ class ServerApp(JupyterApp):
     )
 
     browser = Unicode(
-        u"",
+        "",
         config=True,
         help="""Specify what command to use to invoke a web
                       browser when starting the server. If not specified, the
@@ -1244,7 +1288,7 @@ class ServerApp(JupyterApp):
     tornado_settings = Dict(
         config=True,
         help=_i18n(
-            "Supply overrides for the tornado.web.Application that the " "Jupyter server uses."
+            "Supply overrides for the tornado.web.Application that the Jupyter server uses."
         ),
     )
 
@@ -1292,7 +1336,8 @@ class ServerApp(JupyterApp):
     )
 
     jinja_environment_options = Dict(
-        config=True, help=_i18n("Supply extra arguments that will be passed to Jinja environment.")
+        config=True,
+        help=_i18n("Supply extra arguments that will be passed to Jinja environment."),
     )
 
     jinja_template_vars = Dict(
@@ -1418,11 +1463,15 @@ class ServerApp(JupyterApp):
     )
 
     session_manager_class = Type(
-        default_value=SessionManager, config=True, help=_i18n("The session manager class to use.")
+        default_value=SessionManager,
+        config=True,
+        help=_i18n("The session manager class to use."),
     )
 
     config_manager_class = Type(
-        default_value=ConfigManager, config=True, help=_i18n("The config manager class to use")
+        default_value=ConfigManager,
+        config=True,
+        help=_i18n("The config manager class to use"),
     )
 
     kernel_spec_manager = Instance(KernelSpecManager, allow_none=True)
@@ -1453,6 +1502,13 @@ class ServerApp(JupyterApp):
         help=_i18n("The logout handler class to use."),
     )
 
+    authorizer_class = Type(
+        default_value=AllowAllAuthorizer,
+        klass=Authorizer,
+        config=True,
+        help=_i18n("The authorizer class to use."),
+    )
+
     trust_xheaders = Bool(
         False,
         config=True,
@@ -1590,7 +1646,8 @@ class ServerApp(JupyterApp):
         self.server_extensions = change["new"]
 
     jpserver_extensions = Dict(
-        {},
+        default_value={},
+        value_trait=Bool(),
         config=True,
         help=(
             _i18n(
@@ -1608,6 +1665,29 @@ class ServerApp(JupyterApp):
         help=_i18n("Reraise exceptions encountered loading server extensions?"),
     )
 
+    kernel_ws_protocol = Unicode(
+        None,
+        allow_none=True,
+        config=True,
+        help=_i18n(
+            "Preferred kernel message protocol over websocket to use (default: None). "
+            "If an empty string is passed, select the legacy protocol. If None, "
+            "the selected protocol will depend on what the front-end supports "
+            "(usually the most recent protocol supported by the back-end and the "
+            "front-end)."
+        ),
+    )
+
+    limit_rate = Bool(
+        True,
+        config=True,
+        help=_i18n(
+            "Whether to limit the rate of IOPub messages (default: True). "
+            "If True, use iopub_msg_rate_limit, iopub_data_rate_limit and/or rate_limit_window "
+            "to tune the rate."
+        ),
+    )
+
     iopub_msg_rate_limit = Float(
         1000,
         config=True,
@@ -1753,6 +1833,7 @@ class ServerApp(JupyterApp):
             parent=self,
             log=self.log,
         )
+        self.authorizer = self.authorizer_class(parent=self, log=self.log)
 
     def init_logging(self):
         # This prevents double log messages because tornado use a root logger that
@@ -1839,6 +1920,7 @@ class ServerApp(JupyterApp):
             self.default_url,
             self.tornado_settings,
             self.jinja_environment_options,
+            authorizer=self.authorizer,
         )
         if self.certfile:
             self.ssl_options["certfile"] = self.certfile
@@ -1861,7 +1943,6 @@ class ServerApp(JupyterApp):
             )
             if self.ssl_options.get("ca_certs", False):
                 self.ssl_options.setdefault("cert_reqs", ssl.CERT_REQUIRED)
-            ssl_options = self.ssl_options
 
         self.login_handler_class.validate_security(self, ssl_options=self.ssl_options)
 
@@ -1896,10 +1977,10 @@ class ServerApp(JupyterApp):
             netloc = urlencode_unix_socket_path(self.sock)
         else:
             # Handle nonexplicit hostname.
-            if self.ip in ("", "0.0.0.0"):
+            if self.ip in ("", "0.0.0.0", "::"):
                 ip = "%s" % socket.gethostname()
             else:
-                ip = self.ip
+                ip = "[{}]".format(self.ip) if ":" in self.ip else self.ip
             netloc = "{ip}:{port}".format(ip=ip, port=self.port)
             if self.certfile:
                 scheme = "https"
@@ -1914,7 +1995,12 @@ class ServerApp(JupyterApp):
                 query = urllib.parse.urlencode({"token": token})
         # Build the URL Parts to dump.
         urlparts = urllib.parse.ParseResult(
-            scheme=scheme, netloc=netloc, path=path, params=None, query=query, fragment=None
+            scheme=scheme,
+            netloc=netloc,
+            path=path,
+            params=None,
+            query=query,
+            fragment=None,
         )
         return urlparts
 
@@ -1959,7 +2045,12 @@ class ServerApp(JupyterApp):
         try:
             from jupyter_server.terminal import initialize
 
-            initialize(self.web_app, self.root_dir, self.connection_url, self.terminado_settings)
+            initialize(
+                self.web_app,
+                self.root_dir,
+                self.connection_url,
+                self.terminado_settings,
+            )
             self.terminals_available = True
         except ImportError as e:
             self.log.warning(_i18n("Terminals not available (error was %s)"), e)
@@ -2056,11 +2147,7 @@ class ServerApp(JupyterApp):
         # This enables merging on keys, which we want for extension enabling.
         # Regular config loading only merges at the class level,
         # so each level clobbers the previous.
-        config_paths = jupyter_config_path()
-        if self.config_dir not in config_paths:
-            # add self.config_dir to the front, if set manually
-            config_paths.insert(0, self.config_dir)
-        manager = ExtensionConfigManager(read_config_path=config_paths)
+        manager = ExtensionConfigManager(read_config_path=self.config_file_paths)
         extensions = manager.get_jpserver_extensions()
 
         for modulename, enabled in sorted(extensions.items()):
@@ -2122,7 +2209,8 @@ class ServerApp(JupyterApp):
         self.log.debug("No activity for %d seconds.", seconds_since_active)
         if seconds_since_active > self.shutdown_no_activity_timeout:
             self.log.info(
-                "No kernels or terminals for %d seconds; shutting down.", seconds_since_active
+                "No kernels or terminals for %d seconds; shutting down.",
+                seconds_since_active,
             )
             self.stop()
 
@@ -2214,7 +2302,10 @@ class ServerApp(JupyterApp):
                     else:
                         self.log.info(_i18n("The port %i is already in use.") % port)
                     continue
-                elif e.errno in (errno.EACCES, getattr(errno, "WSAEACCES", errno.EACCES)):
+                elif e.errno in (
+                    errno.EACCES,
+                    getattr(errno, "WSAEACCES", errno.EACCES),
+                ):
                     self.log.warning(_i18n("Permission to listen on port %i denied.") % port)
                     continue
                 else:
@@ -2272,7 +2363,11 @@ class ServerApp(JupyterApp):
 
     @catch_config_error
     def initialize(
-        self, argv=None, find_extensions=True, new_httpserver=True, starter_extension=None
+        self,
+        argv=None,
+        find_extensions=True,
+        new_httpserver=True,
+        starter_extension=None,
     ):
         """Initialize the Server application class, configurables, web application, and http server.
 
@@ -2433,9 +2528,9 @@ class ServerApp(JupyterApp):
         is configured, root_dir will be set to the parent
         directory of file_to_run.
         """
-        rootdir_abspath = pathlib.Path(self.root_dir).resolve()
+        rootdir_abspath = pathlib.Path(self.root_dir).absolute()
         file_rawpath = pathlib.Path(self.file_to_run)
-        combined_path = (rootdir_abspath / file_rawpath).resolve()
+        combined_path = (rootdir_abspath / file_rawpath).absolute()
         is_child = str(combined_path).startswith(str(rootdir_abspath))
 
         if is_child:
@@ -2554,8 +2649,10 @@ class ServerApp(JupyterApp):
 
         assembled_url, _ = self._prepare_browser_open()
 
-        b = lambda: browser.open(assembled_url, new=self.webbrowser_open_new)
-        threading.Thread(target=b).start()
+        def target():
+            browser.open(assembled_url, new=self.webbrowser_open_new)
+
+        threading.Thread(target=target).start()
 
     def start_app(self):
         super(ServerApp, self).start()
@@ -2682,7 +2779,7 @@ class ServerApp(JupyterApp):
                 self.io_loop.add_callback(self._stop)
 
 
-def list_running_servers(runtime_dir=None):
+def list_running_servers(runtime_dir=None, log=None):
     """Iterate over the server info files of running Jupyter servers.
 
     Given a runtime directory, find jpserver-* files in the security directory,
@@ -2709,8 +2806,9 @@ def list_running_servers(runtime_dir=None):
                 # If the process has died, try to delete its info file
                 try:
                     os.unlink(os.path.join(runtime_dir, file_name))
-                except OSError:
-                    pass  # TODO: This should warn or log or something
+                except OSError as e:
+                    if log:
+                        log.warning(_i18n("Deleting server info file failed: %s.") % e)
 
 
 # -----------------------------------------------------------------------------
