@@ -113,7 +113,7 @@ def atomic_writing(path, text=True, encoding="utf-8", log=None, **kwargs):
 
     try:
         yield fileobj
-    except:
+    except BaseException:
         # Failed! Move the backup file back to the real path to avoid corruption
         fileobj.close()
         replace_file(tmp_path, path)
@@ -161,7 +161,7 @@ def _simple_writing(path, text=True, encoding="utf-8", log=None, **kwargs):
 
     try:
         yield fileobj
-    except:
+    except BaseException:
         fileobj.close()
         raise
 
@@ -219,7 +219,7 @@ class FileManagerMixin(Configurable):
         """context manager for turning permission errors into 403."""
         try:
             yield
-        except (OSError, IOError) as e:
+        except OSError as e:
             if e.errno in {errno.EPERM, errno.EACCES}:
                 # make 403 error message without root prefix
                 # this may not work perfectly on unicode paths on Python 2,
@@ -227,7 +227,7 @@ class FileManagerMixin(Configurable):
                 if not os_path:
                     os_path = e.filename or "unknown file"
                 path = to_api_path(os_path, root=self.root_dir)
-                raise HTTPError(403, u"Permission denied: %s" % path) from e
+                raise HTTPError(403, "Permission denied: %s" % path) from e
             else:
                 raise
 
@@ -261,11 +261,13 @@ class FileManagerMixin(Configurable):
             raise HTTPError(404, "%s is outside root contents directory" % path)
         return os_path
 
-    def _read_notebook(self, os_path, as_version=4):
+    def _read_notebook(self, os_path, as_version=4, capture_validation_error=None):
         """Read a notebook from an os path."""
         with self.open(os_path, "r", encoding="utf-8") as f:
             try:
-                return nbformat.read(f, as_version=as_version)
+                return nbformat.read(
+                    f, as_version=as_version, capture_validation_error=capture_validation_error
+                )
             except Exception as e:
                 e_orig = e
 
@@ -277,19 +279,26 @@ class FileManagerMixin(Configurable):
             if not self.use_atomic_writing or not os.path.exists(tmp_path):
                 raise HTTPError(
                     400,
-                    u"Unreadable Notebook: %s %r" % (os_path, e_orig),
+                    "Unreadable Notebook: %s %r" % (os_path, e_orig),
                 )
 
             # Move the bad file aside, restore the intermediate, and try again.
             invalid_file = path_to_invalid(os_path)
             replace_file(os_path, invalid_file)
             replace_file(tmp_path, os_path)
-            return self._read_notebook(os_path, as_version)
+            return self._read_notebook(
+                os_path, as_version, capture_validation_error=capture_validation_error
+            )
 
-    def _save_notebook(self, os_path, nb):
+    def _save_notebook(self, os_path, nb, capture_validation_error=None):
         """Save a notebook to an os_path."""
         with self.atomic_writing(os_path, encoding="utf-8") as f:
-            nbformat.write(nb, f, version=nbformat.NO_CONVERT)
+            nbformat.write(
+                nb,
+                f,
+                version=nbformat.NO_CONVERT,
+                capture_validation_error=capture_validation_error,
+            )
 
     def _read_file(self, os_path, format):
         """Read a non-notebook file.
@@ -334,7 +343,7 @@ class FileManagerMixin(Configurable):
                 b64_bytes = content.encode("ascii")
                 bcontent = decodebytes(b64_bytes)
         except Exception as e:
-            raise HTTPError(400, u"Encoding error saving %s: %s" % (os_path, e)) from e
+            raise HTTPError(400, "Encoding error saving %s: %s" % (os_path, e)) from e
 
         with self.atomic_writing(os_path, text=False) as f:
             f.write(bcontent)
@@ -352,11 +361,18 @@ class AsyncFileManagerMixin(FileManagerMixin):
         """
         await async_copy2_safe(src, dest, log=self.log)
 
-    async def _read_notebook(self, os_path, as_version=4):
+    async def _read_notebook(self, os_path, as_version=4, capture_validation_error=None):
         """Read a notebook from an os path."""
         with self.open(os_path, "r", encoding="utf-8") as f:
             try:
-                return await run_sync(partial(nbformat.read, as_version=as_version), f)
+                return await run_sync(
+                    partial(
+                        nbformat.read,
+                        as_version=as_version,
+                        capture_validation_error=capture_validation_error,
+                    ),
+                    f,
+                )
             except Exception as e:
                 e_orig = e
 
@@ -368,19 +384,29 @@ class AsyncFileManagerMixin(FileManagerMixin):
             if not self.use_atomic_writing or not os.path.exists(tmp_path):
                 raise HTTPError(
                     400,
-                    u"Unreadable Notebook: %s %r" % (os_path, e_orig),
+                    "Unreadable Notebook: %s %r" % (os_path, e_orig),
                 )
 
             # Move the bad file aside, restore the intermediate, and try again.
             invalid_file = path_to_invalid(os_path)
-            async_replace_file(os_path, invalid_file)
-            async_replace_file(tmp_path, os_path)
-            return await self._read_notebook(os_path, as_version)
+            await async_replace_file(os_path, invalid_file)
+            await async_replace_file(tmp_path, os_path)
+            return await self._read_notebook(
+                os_path, as_version, capture_validation_error=capture_validation_error
+            )
 
-    async def _save_notebook(self, os_path, nb):
+    async def _save_notebook(self, os_path, nb, capture_validation_error=None):
         """Save a notebook to an os_path."""
         with self.atomic_writing(os_path, encoding="utf-8") as f:
-            await run_sync(partial(nbformat.write, version=nbformat.NO_CONVERT), nb, f)
+            await run_sync(
+                partial(
+                    nbformat.write,
+                    version=nbformat.NO_CONVERT,
+                    capture_validation_error=capture_validation_error,
+                ),
+                nb,
+                f,
+            )
 
     async def _read_file(self, os_path, format):
         """Read a non-notebook file.
@@ -425,7 +451,7 @@ class AsyncFileManagerMixin(FileManagerMixin):
                 b64_bytes = content.encode("ascii")
                 bcontent = decodebytes(b64_bytes)
         except Exception as e:
-            raise HTTPError(400, u"Encoding error saving %s: %s" % (os_path, e)) from e
+            raise HTTPError(400, "Encoding error saving %s: %s" % (os_path, e)) from e
 
         with self.atomic_writing(os_path, text=False) as f:
             await run_sync(f.write, bcontent)
