@@ -143,7 +143,8 @@ class MappingKernelManager(MultiKernelManager):
         return defaultdict(lambda: {"buffer": [], "session_key": "", "channels": {}})
 
     last_kernel_activity = Instance(
-        datetime, help="The last activity on any kernel, including shutting down a kernel"
+        datetime,
+        help="The last activity on any kernel, including shutting down a kernel",
     )
 
     def __init__(self, **kwargs):
@@ -210,7 +211,9 @@ class MappingKernelManager(MultiKernelManager):
                 kwargs["kernel_id"] = kernel_id
             kernel_id = await ensure_async(self.pinned_superclass.start_kernel(self, **kwargs))
             self._kernel_connections[kernel_id] = 0
-            asyncio.ensure_future(self._finish_kernel_start(kernel_id))
+            fut = asyncio.ensure_future(self._finish_kernel_start(kernel_id))
+            if not getattr(self, "use_pending_kernels", None):
+                await fut
             # add busy/activity markers:
             kernel = self.get_kernel(kernel_id)
             kernel.execution_state = "starting"
@@ -300,7 +303,7 @@ class MappingKernelManager(MultiKernelManager):
         """
 
         if not self.buffer_offline_messages:
-            for channel, stream in channels.items():
+            for _, stream in channels.items():
                 stream.close()
             return
 
@@ -511,13 +514,17 @@ class MappingKernelManager(MultiKernelManager):
             self.last_kernel_activity = kernel.last_activity = utcnow()
 
             idents, fed_msg_list = session.feed_identities(msg_list)
-            msg = session.deserialize(fed_msg_list)
+            msg = session.deserialize(fed_msg_list, content=False)
 
             msg_type = msg["header"]["msg_type"]
             if msg_type == "status":
+                msg = session.deserialize(fed_msg_list)
                 kernel.execution_state = msg["content"]["execution_state"]
                 self.log.debug(
-                    "activity on %s: %s (%s)", kernel_id, msg_type, kernel.execution_state
+                    "activity on %s: %s (%s)",
+                    kernel_id,
+                    msg_type,
+                    kernel.execution_state,
                 )
             else:
                 self.log.debug("activity on %s: %s", kernel_id, msg_type)
@@ -538,7 +545,7 @@ class MappingKernelManager(MultiKernelManager):
         """
         if not self._initialized_culler and self.cull_idle_timeout > 0:
             if self._culler_callback is None:
-                loop = IOLoop.current()
+                _ = IOLoop.current()
                 if self.cull_interval <= 0:  # handle case where user set invalid value
                     self.log.warning(
                         "Invalid value for 'cull_interval' detected (%s) - using default value (%s).",
@@ -582,7 +589,7 @@ class MappingKernelManager(MultiKernelManager):
     async def cull_kernel_if_idle(self, kernel_id):
         kernel = self._kernels[kernel_id]
 
-        if getattr(kernel, "execution_state") == "dead":
+        if getattr(kernel, "execution_state", None) == "dead":
             self.log.warning(
                 "Culling '%s' dead kernel '%s' (%s).",
                 kernel.execution_state,
