@@ -5,13 +5,20 @@ from anyio.to_thread import run_sync
 from tornado import web
 
 from ...base.handlers import APIHandler
+from jupyter_server.auth import authorized
+
+
+AUTH_RESOURCE = "nbconvert"
 
 
 LOCK = asyncio.Lock()
 
 
 class NbconvertRootHandler(APIHandler):
+    auth_resource = AUTH_RESOURCE
+
     @web.authenticated
+    @authorized
     async def get(self):
         try:
             from nbconvert.exporters import base
