@@ -5,11 +5,16 @@ from tornado import web
 
 from . import csp_report_uri
 from ...base.handlers import APIHandler
+from jupyter_server.auth import authorized
+
+
+AUTH_RESOURCE = "csp"
 
 
 class CSPReportHandler(APIHandler):
     """Accepts a content security policy violation report"""
 
+    auth_resource = AUTH_RESOURCE
     _track_activity = False
 
     def skip_check_origin(self):
@@ -21,10 +26,12 @@ class CSPReportHandler(APIHandler):
         return
 
     @web.authenticated
+    @authorized
     def post(self):
         """Log a content security policy violation report"""
         self.log.warning(
-            "Content security violation: %s", self.request.body.decode("utf8", "replace")
+            "Content security violation: %s",
+            self.request.body.decode("utf8", "replace"),
         )
 
 
