@@ -1,143 +1,10 @@
 import inspect
-from traitlets import ClassBasedTraitType, Undefined, warn
+from ast import literal_eval
 
-# Traitlet's 5.x includes a set of utilities for building
-# description strings for objects. Traitlets 5.x does not
-# support Python 3.6, but jupyter_server does; instead
-# jupyter_server uses traitlets 4.3.x which doesn't have
-# this `descriptions` submodule. This chunk in the except
-# clause is a copy-and-paste from traitlets 5.0.5.
-try:
-    from traitlets.utils.descriptions import describe
-except ImportError:
-    import inspect
-    import re
-    import types
-
-    def describe(article, value, name=None, verbose=False, capital=False):
-        """Return string that describes a value
-        Parameters
-        ----------
-        article : str or None
-            A definite or indefinite article. If the article is
-            indefinite (i.e. "a" or "an") the appropriate one
-            will be infered. Thus, the arguments of ``describe``
-            can themselves represent what the resulting string
-            will actually look like. If None, then no article
-            will be prepended to the result. For non-articled
-            description, values that are instances are treated
-            definitely, while classes are handled indefinitely.
-        value : any
-            The value which will be named.
-        name : str or None (default: None)
-            Only applies when ``article`` is "the" - this
-            ``name`` is a definite reference to the value.
-            By default one will be infered from the value's
-            type and repr methods.
-        verbose : bool (default: False)
-            Whether the name should be concise or verbose. When
-            possible, verbose names include the module, and/or
-            class name where an object was defined.
-        capital : bool (default: False)
-            Whether the first letter of the article should
-            be capitalized or not. By default it is not.
-        Examples
-        --------
-        Indefinite description:
-        >>> describe("a", object())
-        'an object'
-        >>> describe("a", object)
-        'an object'
-        >>> describe("a", type(object))
-        'a type'
-
-        Definite description:
-        >>> describe("the", object())  # doctest: +ELLIPSIS
-        "the object at '0x...'"
-        >>> describe("the", object)
-        'the object object'
-        >>> describe("the", type(object))
-        'the type type'
-
-        Definitely named description:
-        >>> describe("the", object(), "I made")
-        'the object I made'
-        >>> describe("the", object, "I will use")
-        'the object I will use'
-        """
-        if isinstance(article, str):
-            article = article.lower()
-
-        if not inspect.isclass(value):
-            typename = type(value).__name__
-        else:
-            typename = value.__name__
-        if verbose:
-            typename = _prefix(value) + typename
-
-        if article == "the" or (article is None and not inspect.isclass(value)):
-            if name is not None:
-                result = "{} {}".format(typename, name)
-                if article is not None:
-                    return add_article(result, True, capital)
-                else:
-                    return result
-            else:
-                tick_wrap = False
-                if inspect.isclass(value):
-                    name = value.__name__
-                elif isinstance(value, types.FunctionType):
-                    name = value.__name__
-                    tick_wrap = True
-                elif isinstance(value, types.MethodType):
-                    name = value.__func__.__name__
-                    tick_wrap = True
-                elif type(value).__repr__ in (object.__repr__, type.__repr__):
-                    name = "at '%s'" % hex(id(value))
-                    verbose = False
-                else:
-                    name = repr(value)
-                    verbose = False
-                if verbose:
-                    name = _prefix(value) + name
-                if tick_wrap:
-                    name = name.join("''")
-                return describe(article, value, name=name,
-                    verbose=verbose, capital=capital)
-        elif article in ("a", "an") or article is None:
-            if article is None:
-                return typename
-            return add_article(typename, False, capital)
-        else:
-            raise ValueError("The 'article' argument should "
-                "be 'the', 'a', 'an', or None not %r" % article)
-
-
-    def add_article(name, definite=False, capital=False):
-        """Returns the string with a prepended article.
-        The input does not need to begin with a charater.
-        Parameters
-        ----------
-        definite : bool (default: False)
-            Whether the article is definite or not.
-            Indefinite articles being 'a' and 'an',
-            while 'the' is definite.
-        capital : bool (default: False)
-            Whether the added article should have
-            its first letter capitalized or not.
-        """
-        if definite:
-            result = "the " + name
-        else:
-            first_letters = re.compile(r'[\W_]+').sub('', name)
-            if first_letters[:1].lower() in 'aeiou':
-                result = 'an ' + name
-            else:
-                result = 'a ' + name
-        if capital:
-            return result[0].upper() + result[1:]
-        else:
-            return result
+from traitlets import ClassBasedTraitType
+from traitlets import TraitError
+from traitlets import Undefined
+from traitlets.utils.descriptions import describe
 
 
 class TypeFromClasses(ClassBasedTraitType):
@@ -198,8 +65,10 @@ class TypeFromClasses(ClassBasedTraitType):
             try:
                 value = self._resolve_string(value)
             except ImportError:
-                raise TraitError("The '%s' trait of %s instance must be a type, but "
-                                 "%r could not be imported" % (self.name, obj, value))
+                raise TraitError(
+                    "The '%s' trait of %s instance must be a type, but "
+                    "%r could not be imported" % (self.name, obj, value)
+                )
         try:
             if self.subclass_from_klasses(value):
                 return value
@@ -213,12 +82,12 @@ class TypeFromClasses(ClassBasedTraitType):
         result = "a subclass of "
         for klass in self.klasses:
             if not isinstance(klass, str):
-                klass = klass.__module__ + '.' + klass.__name__
+                klass = klass.__module__ + "." + klass.__name__
             result += f"{klass} or "
         # Strip the last "or"
-        result = result.strip(" or ")
+        result = result.strip(" or ")  # noqa B005
         if self.allow_none:
-            return result + ' or None'
+            return result + " or None"
         return result
 
     def instance_init(self, obj):
@@ -234,7 +103,7 @@ class TypeFromClasses(ClassBasedTraitType):
                 try:
                     klass = self._resolve_string(klass)
                     self.importable_klasses.append(klass)
-                except:
+                except Exception:
                     pass
             else:
                 self.importable_klasses.append(klass)
@@ -247,7 +116,7 @@ class TypeFromClasses(ClassBasedTraitType):
         if isinstance(value, str):
             return repr(value)
         else:
-            return repr(f'{value.__module__}.{value.__name__}')
+            return repr(f"{value.__module__}.{value.__name__}")
 
 
 class InstanceFromClasses(ClassBasedTraitType):
@@ -255,6 +124,7 @@ class InstanceFromClasses(ClassBasedTraitType):
     The value can also be an instance of a subclass of the specified classes.
     Subclasses can declare default classes by overriding the klass attribute
     """
+
     def __init__(self, klasses=None, args=None, kw=None, **kwargs):
         """Construct an Instance trait.
         This trait allows values that are instances of a particular
@@ -286,8 +156,10 @@ class InstanceFromClasses(ClassBasedTraitType):
         elif all(inspect.isclass(k) or isinstance(k, str) for k in klasses):
             self.klasses = klasses
         else:
-            raise TraitError('The klasses attribute must be a list of class names or classes'
-                                ' not: %r' % klass)
+            raise TraitError(
+                "The klasses attribute must be a list of class names or classes"
+                " not: %r" % klasses
+            )
 
         if (kw is not None) and not isinstance(kw, dict):
             raise TraitError("The 'kw' argument must be a dict or None.")
@@ -317,9 +189,9 @@ class InstanceFromClasses(ClassBasedTraitType):
             else:
                 result += describe("a", klass)
             result += " or "
-        result = result.strip(" or ")
+        result = result.strip(" or ")  # noqa B005
         if self.allow_none:
-            result += ' or None'
+            result += " or None"
         return result
 
     def instance_init(self, obj):
@@ -335,7 +207,7 @@ class InstanceFromClasses(ClassBasedTraitType):
                 try:
                     klass = self._resolve_string(klass)
                     self.importable_klasses.append(klass)
-                except:
+                except Exception:
                     pass
             else:
                 self.importable_klasses.append(klass)
@@ -343,11 +215,10 @@ class InstanceFromClasses(ClassBasedTraitType):
     def make_dynamic_default(self):
         if (self.default_args is None) and (self.default_kwargs is None):
             return None
-        return self.klass(*(self.default_args or ()),
-                          **(self.default_kwargs or {}))
+        return self.klass(*(self.default_args or ()), **(self.default_kwargs or {}))
 
     def default_value_repr(self):
         return repr(self.make_dynamic_default())
 
     def from_string(self, s):
-        return _safe_literal_eval(s)
+        return literal_eval(s)
